/*:
 * @target MZ
 * @plugindesc プラグイン：属性有効度の確率発動
 * @author GrayOgre
 * @help
 * 属性有効度を指定された確率で反映します。
 * 
 * 属性有効度を設定できるオブジェクトのメモに
 * <resistProbability:属性,確率(%)> と記載することで
 * 指定された属性の属性有効度が指定された確率でダメージに反映されます。
 * 対象となる属性有効度は通常の方法で特徴に設定しておく必要があります。
 * 
 * 例：炎属性の有効度を30%の確率で有効にする。
 * <resistProbability:炎,30>
 * 
 * 〈確率発動の無効化〉
 * メモに <cancelProbability> と記載ししたスキルは、
 * 上記の確率発動する属性有効度を無視してダメージを与えます。
 * 確率発動ではない属性有効度は、このスキルにおいても効果を与えます。
 * 
 * プラグインコマンドはありません。
 *
 * var 0.9.1
 * 
 * Copyright (c) 2021 GrayOgre
 * Released under the MIT license
 * https://opensource.org/licenses/mit-license.php
 *
 * @orderAfter PluginCommonBase
 * 
 */

(() => {
    'use strict';

    const probRegExpHead = "<resistProbability:";
    const probRegExpTeil = ",(\\d+)>";
    const cancelResistProbabilityKey = "cancelProbability";

    function calcElementRateWithProbability(trait, elementId, invalid) {
        if (trait._elementProbabilityGORP === undefined) {
            trait._elementProbabilityGORP = new Array($dataSystem.elements.length);
        }
        if (trait._elementProbabilityGORP[elementId] === undefined) {
            const re = new RegExp(probRegExpHead + $dataSystem.elements[elementId] + probRegExpTeil);
            const match = re.exec(trait.objGORP.note);
            if (match) {
                trait._elementProbabilityGORP[elementId] = parseInt(match[1]) / 100;
            } else {
                trait._elementProbabilityGORP[elementId] = 1;
            }
        }
        const prob = trait._elementProbabilityGORP[elementId];
        const result = (!invalid && prob > Math.random()) ? trait.value : 1;
        return result;
    }   

    const _Game_BattleBase_alltraits = Game_BattlerBase.prototype.allTraits
    Game_BattlerBase.prototype.allTraits = function() {
        this.traitObjects().forEach(obj => obj.traits.forEach(trait => trait.objGORP = obj));
        return _Game_BattleBase_alltraits.call(this);
    }

    Game_BattlerBase.prototype.elementRate = function(elementId) {
        let r = 1;
        for (const trait of this.traitsWithId(Game_BattlerBase.TRAIT_ELEMENT_RATE, elementId)) {
            r *= calcElementRateWithProbability(trait, elementId, this.invalidProbabilityGORP);
        }
        return r;
    };

    const _Game_Action_makeDamageValue = Game_Action.prototype.makeDamageValue;
    Game_Action.prototype.makeDamageValue = function(target, critical) {
        const item = this.item();
        if (item.invalidProbabilityGORP === undefined) {
            item.invalidProbabilityGORP = this.isSkill() && item.meta[cancelResistProbabilityKey] !== undefined;
        }
        target.invalidProbabilityGORP = item.invalidProbabilityGORP;
        return _Game_Action_makeDamageValue.call(this, target, critical);
    }
})();